



#include "DriveInfo.h"

int iNumOfDrives;			// number of total installed drives 
TCHAR szDrives[26][10];

void InitDriveInfo()
{
	iNumOfDrives = 0;

	// get number of installed drives
	/*
	DWORD dwDriveMask = GetLogicalDrives();
	for (int iDrive = 0; iDrive < 32; iDrive++)
	{
		if (dwDriveMask & 1)
		{
			iNumOfDrives++;

		}
		dwDriveMask >>= 1;
	}*/

	ZeroMemory(szDrives, sizeof(szDrives));

	//** get all used drive letters

	TCHAR szDrivebuf[MAX_PATH];		// will hold all drive letters in unicode  
	memset(szDrivebuf, 0, MAX_PATH * sizeof(TCHAR) );
	GetLogicalDriveStrings(MAX_PATH, szDrivebuf);
	TCHAR* p = szDrivebuf;
	do
	{
		_tcscpy_s(szDrives[iNumOfDrives++], 10, p);
		// go to next drive by moving to next NULL character
		while (*p++);
	}
	while (*p);
}

/*
 * Returns the number of total installed drives for the system
 */
int GetNumOfInstalledDrives() { return iNumOfDrives; }

/*
 * Returns the drive letter of the specified drive. Returns FALSE if error occurs.
 */
BOOL GetDriveLetter(int driveIndex, TCHAR* szDriveLetter)
{
	if (driveIndex < 0 || driveIndex >= iNumOfDrives) 
		return FALSE;
	
	//*szDriveLetter = (TCHAR*)(&szDrives[driveIndex]);
	_tcscpy_s(szDriveLetter, 5, szDrives[driveIndex]);
	return TRUE;
}


/*
 * Read volume name and serial for the specified drive. Returns FALSE if error occurs.
 */
BOOL GetDriveVolumeName(int driveIndex, TCHAR* szDriveName, int iDriveNameLength)
{
	DWORD dwSerialNumber;				// serial number of drive
	DWORD dwSystemFlags;				// receives flags for drive
	TCHAR szFileSystem[MAX_PATH];		// file system name
	DWORD dwCompLength;

	if (driveIndex < 0 || driveIndex >= iNumOfDrives) 
		return FALSE;

	if (GetVolumeInformation(szDrives[driveIndex], szDriveName, iDriveNameLength,
						 &dwSerialNumber, &dwCompLength, &dwSystemFlags, szFileSystem, MAX_PATH))
	{
		return TRUE;
	}
	else
		return FALSE;
}

/*
 * Get the used File System. Returns FALSE if error occurs.
 */
BOOL GetDriveFileSystemName(int driveIndex, TCHAR* szFileSystemName, int iFileSystemNameLength)
{
	DWORD dwSerialNumber;				// serial number of drive
	DWORD dwSystemFlags;				// receives flags for drive
	TCHAR szDriveName[MAX_PATH];		// file system name
	DWORD dwCompLength;

	if (driveIndex < 0 || driveIndex >= iNumOfDrives) 
		return FALSE;

	if (GetVolumeInformation(szDrives[driveIndex], szDriveName, MAX_PATH,
						 &dwSerialNumber, &dwCompLength, &dwSystemFlags, szFileSystemName, iFileSystemNameLength))
	{
		return TRUE;
	}
	else
		return FALSE;
}

/*
 * Get the used File System as constant.
 */
FILESYSTEMTYPE GetDriveFileSystem(int driveIndex)
{
	DWORD dwSerialNumber;				// serial number of drive
	DWORD dwSystemFlags;				// receives flags for drive
	TCHAR szDriveName[MAX_PATH];		// file system name
	DWORD dwCompLength;
	TCHAR szFileSystemName[MAX_PATH];
	
	if (driveIndex < 0 || driveIndex >= iNumOfDrives) 
		return FS_UNKNOWN;

	ZeroMemory(szDriveName, MAX_PATH * sizeof(TCHAR));
	if (GetVolumeInformation(szDrives[driveIndex], szDriveName, MAX_PATH,
						 &dwSerialNumber, &dwCompLength, &dwSystemFlags, szFileSystemName, MAX_PATH))
	{
		if (_tcsicmp(szFileSystemName, _T("ntfs")) == 0)
			return FS_NTFS;
		else if (_tcsicmp(szFileSystemName, _T("fat")) == 0)
			return FS_FAT;
		else if (_tcsicmp(szFileSystemName, _T("fat32")) == 0)
			return FS_FAT32;
		else 
			return FS_UNKNOWN;
	}
	else
		return FS_UNKNOWN;

}


/*
 * Get the serial number of the drive. Returns FALSE if error occurs.
 */
BOOL GetDriveSerialNumber(int driveIndex, DWORD* dwSerialNumber)
{
	DWORD dwSystemFlags;				// receives flags for drive
	TCHAR szDriveName[MAX_PATH];		// file system name
	DWORD dwCompLength;
	TCHAR szFileSystemName[MAX_PATH];

	if (driveIndex < 0 || driveIndex >= iNumOfDrives) 
		return FALSE;

	if (GetVolumeInformation(szDrives[driveIndex], szDriveName, MAX_PATH,
						 dwSerialNumber, &dwCompLength, &dwSystemFlags, szFileSystemName, MAX_PATH))
	{
		return TRUE;
	}
	else
		return FALSE;
}

/*
 * returns the type of the drive as a string
 */
BOOL GetDriveTypeString(int driveIndex, TCHAR* szDriveType, int iDriveTypeLength)
{
	if (driveIndex < 0 || driveIndex >= iNumOfDrives) 
		return FALSE;

	switch (GetDriveType(szDrives[driveIndex]))
	{
	case DRIVE_NO_ROOT_DIR:
	case DRIVE_UNKNOWN:
		{
			_tcscpy_s(szDriveType, iDriveTypeLength, _T("Unknown Drive Type"));
		}
		break;

	case DRIVE_REMOVABLE:
		{
			_tcscpy_s(szDriveType, iDriveTypeLength, _T("Removable Drive"));
		}
		break;

	case DRIVE_FIXED:
		{
			_tcscpy_s(szDriveType, iDriveTypeLength, _T("Fixed Drive"));
		}
		break;

	case DRIVE_REMOTE:
		{
			_tcscpy_s(szDriveType, iDriveTypeLength, _T("Network Drive"));
		}
		break;

	case DRIVE_CDROM:
		{
			_tcscpy_s(szDriveType, iDriveTypeLength, _T("CDROM Drive"));
		}
		break;

	case DRIVE_RAMDISK:
		{
			_tcscpy_s(szDriveType, iDriveTypeLength, _T("RAM Disk"));
		}
		break;

	default:break;
	}

	return TRUE;
}



BOOL GetDisksProperty(int driveIndex, PSTORAGE_DEVICE_DESCRIPTOR pDevDesc)
{
	STORAGE_PROPERTY_QUERY Query;		// input param for query
	DWORD dwOutBytes;					// IOCTL output length
	BOOL bResult;						// IOCTL return val
	HANDLE hDevice;						// handle to device to open
	TCHAR szDriveToOpen[15];

	// first make the correct volume name passed to createfile, e.g. "\\\\.\\c:\"
	ZeroMemory(szDriveToOpen, 15 * sizeof(TCHAR));
	_tcscat_s(szDriveToOpen, 15, _T("\\\\.\\"));
	_tcsncat_s(szDriveToOpen, 15, szDrives[driveIndex], 2);		//only copy letter and ':', but not the backslash
	
	// open the volume
	hDevice = CreateFile(szDriveToOpen, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, 
					NULL, OPEN_EXISTING, NULL, NULL);
	if (hDevice == INVALID_HANDLE_VALUE) 
		return FALSE;

	// specify the query type
	Query.PropertyId = StorageDeviceProperty;
	Query.QueryType = PropertyStandardQuery;

	 // Query using IOCTL_STORAGE_QUERY_PROPERTY 
	bResult = DeviceIoControl(	hDevice, // device handle
								IOCTL_STORAGE_QUERY_PROPERTY, // info of device property
								&Query, sizeof(STORAGE_PROPERTY_QUERY), // input data buffer
								pDevDesc, pDevDesc->Size, // output data buffer
								&dwOutBytes, // out's length
								(LPOVERLAPPED)NULL); 
	CloseHandle(hDevice);
	return bResult;
}

/*
 * Converts the given bus type to a string.
 */
void AddBusTypeStr(STORAGE_BUS_TYPE bustype, TCHAR* szBusType, int iBusTypeLength)
{
	switch(bustype)
	{
	case BusTypeScsi: { _tcscpy_s(szBusType, iBusTypeLength, _T("SCSI Bus")); } break;
	case BusTypeAtapi : { _tcscpy_s(szBusType, iBusTypeLength, _T("ATAPI Bus")); } break;
	case BusTypeAta  : { _tcscpy_s(szBusType, iBusTypeLength, _T("ATA Bus")); } break;
	case BusTypeSsa    : { _tcscpy_s(szBusType, iBusTypeLength, _T("SSA Bus")); } break;
	case BusTypeFibre  : { _tcscpy_s(szBusType, iBusTypeLength, _T("Fibre Channel Bus")); } break;
	case BusTypeUsb    : { _tcscpy_s(szBusType, iBusTypeLength, _T("USB Bus")); } break;
	case BusTypeRAID   : { _tcscpy_s(szBusType, iBusTypeLength, _T("RAID Bus")); } break;
	case BusTypeiScsi  : { _tcscpy_s(szBusType, iBusTypeLength, _T("iSCSI Bus")); } break;
	case BusTypeSas   : { _tcscpy_s(szBusType, iBusTypeLength, _T("Serial-Attached SCSI Bus")); } break;
	case BusTypeSata    : { _tcscpy_s(szBusType, iBusTypeLength, _T("Serial ATA Bus")); } break;
	case 0x12     : { _tcscpy_s(szBusType, iBusTypeLength, _T("Secure Digital Bus")); } break;
	case 0x13      : { _tcscpy_s(szBusType, iBusTypeLength, _T("Multimedia Card Bus")); } break;
	default: { _tcscpy_s(szBusType, iBusTypeLength, _T("Unknown Bus Type")); } break;
	}

}

/*
 * Dump first sector (512 bytes) of specified into given memory location.
 * driveIndex: which drive to dump from, in alphabetical order of all available ones
 * pBootSector: byte array which will hold the read data. Must be at least 512 bytes.
 */
BOOL ReadBootSector(int driveIndex, BYTE* pBootSector)
{
	HANDLE hDevice;	
	TCHAR szDriveToOpen[15];
	DWORD dwBytesRead;

	// first make the correct volume name passed to createfile, e.g. "\\\\.\\c:\"
	ZeroMemory(szDriveToOpen, 15 * sizeof(TCHAR));
	_tcscat_s(szDriveToOpen, 15, _T("\\\\.\\"));
	_tcsncat_s(szDriveToOpen, 15, szDrives[driveIndex], 2);		//only copy letter and ':', but not the backslash
	
	// open the volume
	hDevice = CreateFile(szDriveToOpen, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, 
					NULL, OPEN_EXISTING, NULL, NULL);
	if (hDevice == INVALID_HANDLE_VALUE) 
		return FALSE;

	if (ReadFile(hDevice, pBootSector, 512, &dwBytesRead, NULL))
	{
		CloseHandle(hDevice);
		return TRUE;
	}
	else
	{
		CloseHandle(hDevice);
		return FALSE;
	}
}
