

#include "DriveInfoList.h"


	HWND hInfoList;					// handle to information listbox


/**************************
 * Basic Logging functions
 **************************/

void InitLog(HWND hListbox)
{
	hInfoList = hListbox;
}

void AddLog(TCHAR s)
{
	SendMessage(hInfoList, LB_INSERTSTRING, -1, (LPARAM)s);
}

void Add(TCHAR* lpFmt, ...)
{
	TCHAR		buf[MAX_PATH];
	va_list		list;
	va_start(list, lpFmt);
	vswprintf_s(buf, MAX_PATH, lpFmt, list); 
	va_end(list);

	SendMessage(hInfoList, LB_INSERTSTRING, -1 , (LPARAM)buf);
}

void ClearLog()
{
	SendMessage(hInfoList, LB_RESETCONTENT, 0, 0);
}

void SetLogFont()
{
	HFONT hListFont = CreateFont(-11, 0, 0, 0, 200, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS,
                           CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, _T("Courier New"));
	SendMessage(hInfoList, WM_SETFONT, (WPARAM)hListFont, 0);
}





/********************************
 * Bootsector Information Logging
 ********************************/



void AddBootSectorInfos(PNTFS_BOOTSECTOR bs)
{
	Add(_T("Jump instruction to boot code: %X %X %X"), bs->BS_jmpBoot[0], bs->BS_jmpBoot[1], bs->BS_jmpBoot[2]);

	// convert ascii oemname to wchar
	TCHAR OEMName_Unicode[9];
	ZeroMemory(OEMName_Unicode, 8 * sizeof(TCHAR));
	MultiByteToWideChar(CP_ACP, 0, (LPCSTR)&bs->BS_OEMName, -1, OEMName_Unicode, 9);
	// make sure string is zero-terminated
	OEMName_Unicode[8] = 0;
	Add(_T("OEM Name: %s"), OEMName_Unicode);

	// ** BPB infos
	Add(_T("Bytes Per Sector: 0x%X (%d)"), (WORD)bs->BPB_BytesPerSec, bs->BPB_BytesPerSec);
	Add(_T("Sectors Per Cluster: 0x%X (%d)"), bs->BPB_SectorsPerCluster, bs->BPB_SectorsPerCluster);
	Add(_T("Media Descriptor: 0x%X (%d)"), bs->BPB_MediaDescriptor, bs->BPB_MediaDescriptor);
	Add(_T("Number of Heads: 0x%X (%d)"), bs->BPB_NumOfHeads, bs->BPB_NumOfHeads);
	Add(_T("Hidden Sectors: 0x%X (%d)"), bs->BPB_HiddenSectors, bs->BPB_HiddenSectors);
	// ** BPB infos
	Add(_T("Total Sectors: 0x%llX"), bs->BPBEx_TotalSectors, bs->BPBEx_TotalSectors);
	Add(_T("Logical Cluster Number for the file $MFT: 0x%llX"), bs->BPBEx_NumOfLogicalClusters, bs->BPBEx_NumOfLogicalClusters);
	Add(_T("Logical Cluster Number Mirror: 0x%llX"), bs->BPBEx_NumOfLogicalClustersMirr, bs->BPBEx_NumOfLogicalClustersMirr);
	Add(_T("Clusters per File Record: 0x%X (%d)"), bs->BPBEx_ClustersPerFile, bs->BPBEx_ClustersPerFile);
	Add(_T("Clusters per Index Block: 0x%X (%d)"), bs->BPBEx_ClustersPerIndexBlock, bs->BPBEx_ClustersPerIndexBlock);
	Add(_T("Volume Serial Number: 0x%llX"), bs->BPBEx_VolumeSerial, bs->BPBEx_VolumeSerial);
	Add(_T("Checksum: 0x%X (%d)"), bs->BPBEx_Checksum, bs->BPBEx_Checksum);

	WORD* b = (WORD*)bs;
	Add(_T("Last 2 Bytes: %04X"), b[255]);
}

/* 
 * Log the bootsector infos of FAT12/16/32 which they have in common and at the same position.
 * To log the common bootsector values of FAT32 structures, just cast them to FAT when passing to this function.
 */
void AddCommonFATInfos(PFAT_BOOTSECTOR bs)
{
	Add(_T("Jump instruction to boot code: %X %X %X"), bs->BS_jmpBoot[0], bs->BS_jmpBoot[1], bs->BS_jmpBoot[2]);

	// convert ascii oemname to wchar
	TCHAR OEMName_Unicode[9];
	ZeroMemory(OEMName_Unicode, 8 * sizeof(TCHAR));
	MultiByteToWideChar(CP_ACP, 0, (LPCSTR)&bs->BS_OEMName, -1, OEMName_Unicode, 9);
	// make sure string is zero-terminated
	OEMName_Unicode[8] = 0;
	Add(_T("OEM Name: %s"), OEMName_Unicode);

	Add(_T("Bytes Per Sector: 0x%X (%d)"), (WORD)bs->BPB_BytesPerSec, bs->BPB_BytesPerSec);
	Add(_T("Sectors Per Cluster: 0x%X (%d)"), bs->BPB_SectorsPerCluster, bs->BPB_SectorsPerCluster);
	Add(_T("# FAT Data Structures: 0x%X (%d)"), bs->BPB_NumOfCopiesFAT, bs->BPB_NumOfCopiesFAT);
	Add(_T("# 32-byte directory entries: 0x%X (%d)"), bs->BPB_MaxRootDirEntries, bs->BPB_MaxRootDirEntries);
	Add(_T("# Total sectors 16-Bit : 0x%X (%d)"), bs->BPB_NumOfSectors32MB, bs->BPB_NumOfSectors32MB);
	Add(_T("Media Descriptor: 0x%X (%d)"), bs->BPB_MediaDescriptor, bs->BPB_MediaDescriptor);
	Add(_T("# Sectors occupied by 1 FAT: 0x%X (%d)"), bs->BPB_SectorsPerFAT, bs->BPB_SectorsPerFAT);
	Add(_T("# Sectors per Track (int 0x13): 0x%X (%d)"), bs->BPB_SectorsPerTrack, bs->BPB_SectorsPerTrack);
	Add(_T("# Heads (int 0x13): 0x%X (%d)"), bs->BPB_NumOfHeads, bs->BPB_NumOfHeads);
	Add(_T("# Hidden Sectors: 0x%X (%d)"), bs->BPB_NumOfHiddenSectors, bs->BPB_NumOfHiddenSectors);
	Add(_T("# Sectors: 0x%X"), bs->BPB_NumOfSectors, bs->BPB_NumOfSectors);
}


void AddBootSectorInfos(PFAT_BOOTSECTOR bs)
{
	AddCommonFATInfos(bs);

	// FAT16/32 Specific
	Add(_T("Int0x13 Drive Number: 0x%X"), bs->BS_DrvNum, bs->BS_DrvNum);
	Add(_T("Extended Boot Signature: 0x%X"), bs->BS_BootSig, bs->BS_BootSig);
	Add(_T("Volume Serial Number: 0x%X"), bs->BS_VolID, bs->BS_VolID);
	
	// Volume Label - ascii->wchar conversion
	TCHAR VolLab_Unicode[12];
	ZeroMemory(VolLab_Unicode, 11 * sizeof(TCHAR));
	MultiByteToWideChar(CP_ACP, 0, (LPCSTR)&bs->BS_VolLab, -1, VolLab_Unicode, 12);
	VolLab_Unicode[11] = 0;		// make sure string is zero-terminated
	Add(_T("Volume Label: %s"), VolLab_Unicode);

	// File System String - ascii->wchar conversion
	TCHAR FileSys_Unicode[9];
	ZeroMemory(FileSys_Unicode, 8 * sizeof(TCHAR));
	MultiByteToWideChar(CP_ACP, 0, (LPCSTR)&bs->BS_FilSysType, -1, FileSys_Unicode, 9);
	FileSys_Unicode[8] = 0;		// make sure string is zero-terminated
	Add(_T("FileSys String: %s"), FileSys_Unicode);
}


void AddBootSectorInfos(PFAT32_BOOTSECTOR bs)
{
	AddCommonFATInfos((PFAT_BOOTSECTOR)bs);

	Add(_T("# Sectors occupied by 1 FAT (32Bit): 0x%X (%d)"), bs->BPB_FATSz32, bs->BPB_FATSz32);
	Add(_T("Extended Flags: 0x%X (%d)"), bs->BPB_ExtFlags, bs->BPB_ExtFlags);
	Add(_T("FS Version: 0x%X "), bs->BPB_FSVer);
	Add(_T("Root Cluster: 0x%X "), bs->BPB_RootClus);
	Add(_T("FS Info structure: 0x%X "), bs->BPB_FSInfo);
	Add(_T("Sector number of boot record copy: 0x%X (%d) "), bs->BPB_BkBootSec, bs->BPB_BkBootSec);
	// from here on same as FAT12/16, but remember that the values are stored at a different position
	// inside FAT32 structure
	Add(_T("Int0x13 Drive Number: 0x%X"), bs->BS_DrvNum, bs->BS_DrvNum);
	Add(_T("Extended Boot Signature: 0x%X"), bs->BS_BootSig, bs->BS_BootSig);
	Add(_T("Volume Serial Number: 0x%X"), bs->BS_VolID, bs->BS_VolID);
	
	// Volume Label - ascii->wchar conversion
	TCHAR VolLab_Unicode[12];
	ZeroMemory(VolLab_Unicode, 11 * sizeof(TCHAR));
	MultiByteToWideChar(CP_ACP, 0, (LPCSTR)&bs->BS_VolLab, -1, VolLab_Unicode, 12);
	VolLab_Unicode[11] = 0;		// make sure string is zero-terminated
	Add(_T("Volume Label: %s"), VolLab_Unicode);

	// File System String - ascii->wchar conversion
	TCHAR FileSys_Unicode[9];
	ZeroMemory(FileSys_Unicode, 8 * sizeof(TCHAR));
	MultiByteToWideChar(CP_ACP, 0, (LPCSTR)&bs->BS_FilSysType, -1, FileSys_Unicode, 9);
	FileSys_Unicode[8] = 0;		// make sure string is zero-terminated
	Add(_T("FileSys String: %s"), FileSys_Unicode);
}



void AddBootSectorInfos(int iDriveIndex, BYTE* bBootsector)
{
	switch (GetDriveFileSystem(iDriveIndex))
	{
	case FS_NTFS:
		{
			AddBootSectorInfos((PNTFS_BOOTSECTOR)bBootsector);
		}
		break;

	case FS_FAT:
		{
			AddBootSectorInfos((PFAT_BOOTSECTOR)bBootsector);
		}
		break;

	case FS_FAT32:
		{
			AddBootSectorInfos((PFAT32_BOOTSECTOR)bBootsector);
		}
		break;

	default: break;
	}

	return;
}