/* Bootsector Info Tool
 * ---------------------
 * This little tool can extract the bootsector of harddisk/usb drives and save it to a file.
 *
 * Version: 0.6
 * 
 * Sunshine, June 2k9
 * www.sunshine2k.de | www.bastian-molkenthin.de
 */

#include <windows.h>
#include <tchar.h>
#include <stdarg.h>
#include <stdio.h> 
#include <commctrl.h>
#include "DriveInfo.h"
#include "DriveInfoList.h"
#include "resource.h"


/*
 * Global Variables
 */

HWND hDriveComboList;			// handle to drive combobox
BYTE bBootSec[512];				// buffer of last read boot sector
BOOL bBootSecRead = FALSE;		// flag if boot sector was read succesfully

// filename filter string for save/open filedialogs
TCHAR* filterstring = _T("HEX Files (*.hex)\0*.hex\0All Files (*.*)\0*.*\0\0");



/*
 * Add all devices to the combobox drive list
 */
void AddDrivesToCombobox()
{
	TCHAR szDriveVolumeName[MAX_PATH];
	TCHAR szDriveLetter[5];
	TCHAR szComboboxEntry[MAX_PATH];

	// loop over all detected drives
	for (int i = 0; i < GetNumOfInstalledDrives(); i++)
	{
		// reinit strings
		ZeroMemory(szDriveVolumeName, sizeof(TCHAR) * MAX_PATH);
		ZeroMemory(szDriveLetter, sizeof(TCHAR) * 5);
		ZeroMemory(szComboboxEntry, sizeof(TCHAR) * MAX_PATH);

		// build combobox item string if information could be retrieved
		if (GetDriveLetter(i, szDriveLetter))
		{
			_tcscat_s(szComboboxEntry, MAX_PATH, szDriveLetter);
			if (GetDriveVolumeName(i, szDriveVolumeName, MAX_PATH))
			{
				_tcscat_s(szComboboxEntry, MAX_PATH, _T(" ("));
				_tcscat_s(szComboboxEntry, MAX_PATH, szDriveVolumeName);
				_tcscat_s(szComboboxEntry, MAX_PATH, _T(")"));
			}
		}
		SendMessage(hDriveComboList, CB_INSERTSTRING, -1, (LPARAM)szComboboxEntry);
	}
}


/*
 * Add information of selected drive to info listbox
 */
void AddDriveInfoToList()
{
	TCHAR szTmpBuffer[MAX_PATH];	// string of the selected combobox item
	DWORD dwTmp;

	// check if selection is valid
	int iSelectedIndex = (int)SendMessage(hDriveComboList, CB_GETCURSEL, 0, 0);
	if (iSelectedIndex != CB_ERR)
	{
		// first clear infobox
		ClearLog();

		// add Drive Letter
		ZeroMemory(szTmpBuffer, MAX_PATH * sizeof(TCHAR));
		if (GetDriveLetter(iSelectedIndex, szTmpBuffer) != NULL)
		{
			Add(_T("Drive Letter: \t%s"), szTmpBuffer);
		}

		// add Volume Name
		ZeroMemory(szTmpBuffer, MAX_PATH * sizeof(TCHAR));
		if (GetDriveVolumeName(iSelectedIndex, szTmpBuffer, MAX_PATH))
		{
			Add(_T("Volume Name: \t%s"), szTmpBuffer);
		}

		// add drive's file system 
		ZeroMemory(szTmpBuffer, MAX_PATH * sizeof(TCHAR));
		if (GetDriveFileSystemName(iSelectedIndex, szTmpBuffer, MAX_PATH))
		{
			Add(_T("File System: \t%s"), szTmpBuffer);
		}

		// add serial number
		if (GetDriveSerialNumber(iSelectedIndex, &dwTmp))
		{
			Add(_T("Serial Number: \t%X"), dwTmp);
		}

		// add drive type
		if (GetDriveTypeString(iSelectedIndex, szTmpBuffer, MAX_PATH))
		{
			Add(_T("Drive Type: \t%s"), szTmpBuffer);
		}

		// add bus type
		STORAGE_DEVICE_DESCRIPTOR stdevdesc;
		ZeroMemory(&stdevdesc, sizeof(STORAGE_DEVICE_DESCRIPTOR));
		stdevdesc.Size = sizeof(STORAGE_DEVICE_DESCRIPTOR);
		if (GetDisksProperty(iSelectedIndex, &stdevdesc))
		{
			ZeroMemory(szTmpBuffer, MAX_PATH * sizeof(TCHAR));
			AddBusTypeStr(stdevdesc.BusType, szTmpBuffer, MAX_PATH);
			Add(_T("Bus Type: \t%s"), szTmpBuffer);
		}
	}

}



/*
 * Read BootSector to memory buffer
 */
void ReadBootSector()
{
	int iSelectedIndex;

	// first get selected device in list
	iSelectedIndex = (int)SendMessage(hDriveComboList, CB_GETCURSEL, 0, 0);
	
	if (iSelectedIndex == CB_ERR)
	{
		MessageBox(0, _T("No Device selected in list!"), _T("Which one?"), MB_OK);
		return;
	}

	// read boot sector data
	ZeroMemory(&bBootSec, 512);
	if (!ReadBootSector(iSelectedIndex, bBootSec))
	{
		MessageBox(0, _T("Dumping Boot Sector failed"), _T("Error"), MB_ICONINFORMATION);
		bBootSecRead = FALSE;
		return;
	}

	// at this point we have successfully the boot sector in memory
	bBootSecRead = TRUE;
	
	// Show infos
	Add(_T("\n"));
	Add(_T("****** Boot Sector Information *****"));
	AddBootSectorInfos(iSelectedIndex, bBootSec);

	return;

}

/*
 * Save dumped Boot Sector to file
 */
void SaveBootSector(HWND hwnd)
{
	OPENFILENAME ofn;
	TCHAR filename[512];
	HANDLE hFile;
	DWORD dwBytesWritten;

	ZeroMemory(filename, 512 * sizeof(TCHAR));
	ZeroMemory(&ofn, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = hwnd;
	ofn.lpstrFilter = filterstring;	
	ofn.lpstrFile = filename;	
	ofn.nMaxFile =  512;
	ofn.lpstrTitle = _T("Choose file...");
	ofn.Flags = OFN_LONGNAMES | OFN_EXPLORER;
	if (GetSaveFileName(&ofn))
	{			
		hFile = CreateFile(filename, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ, NULL,
							CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
		if (hFile == INVALID_HANDLE_VALUE)
		{
			MessageBox(0, _T("Cannot open file"), _T("Error"), MB_ICONERROR);
			return;
		}

		if (WriteFile(hFile, bBootSec, 512, &dwBytesWritten, NULL))
		{
			MessageBox(0, _T("File saved succesfully"), _T("Error"), MB_OK);
		}
		else
		{
			MessageBox(0, _T("Saving Boot Sector to file failed"), _T("Error"), MB_ICONERROR);
		}

		CloseHandle(hFile);
	}
}


INT_PTR CALLBACK DialogProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{

	case WM_INITDIALOG:
		{
			// set icon
			HICON   hIcon;
			hIcon = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_ICON1));
			if (hIcon) 
				SendMessage(hwnd, WM_SETICON, ICON_SMALL, (LPARAM)hIcon);
			
			// save handle of drive combobox
			hDriveComboList = GetDlgItem(hwnd, ID_DRIVECOMBO);
			
			// init log listbox
			InitLog(GetDlgItem(hwnd, ID_INFOLIST));
			SetLogFont();

			// int drive helper functions
			InitDriveInfo();
			Add(_T("Number of drives detected: %d"), GetNumOfInstalledDrives());
		
			// add all found drives to combobox
			AddDrivesToCombobox();
		}
		break;


	case WM_CLOSE:
		{
			EndDialog(hwnd, 0);
		}
		break;

	case WM_COMMAND:
		{
			switch (HIWORD(wParam))
			{
			case CBN_SELCHANGE:
				{

					switch (LOWORD(wParam))
					{
					case ID_DRIVECOMBO:
						{
							AddDriveInfoToList();
						}
						break;
				
					default: break;
					}
				}
			}

			switch (wParam)
			{
			case ID_READBOOTSECBUTTON:
				{
					ReadBootSector();
				}
				break;

			case ID_SAVEBOOTSECBUTTON:
				{
					SaveBootSector(hwnd);
				}
				break;

			case ID_ABOUTBUTTON:
				{
					MessageBox(hwnd, _T("Bootsector Info Tool\n\nVersion: 0.6\nRefer to readme.txt for further details\n\nSunshine, June 2k9 \nwww.sunshine2k.de | www.bastian-molkenthin.de"),
									 _T("About this tiny app..."),
									 MB_ICONINFORMATION | MB_OK);
				}
				break;

			default:break;
			}
			
		}
		break;

	default: break;
	}

	return FALSE;
}



/*
 * Main Entrypoint
 */
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	DialogBox(hInstance, MAKEINTRESOURCE(100), NULL, DialogProc);
	return 0;
}